@@ -4,12 +4,10 @@
 #ifdef HAVE_SIGNALFD
 static void sig_usr1(int signo) {}
 
-TEST(os_signal, swSignalfd_set) {
+TEST(os_signal, signalfd) {
     int ret;
     sigset_t curset;
 
-    SwooleG.use_signalfd = 1;
-
     swoole_event_init(SW_EVENTLOOP_WAIT_EXIT);
 
     sigemptyset(&curset);
@@ -17,17 +15,16 @@ TEST(os_signal, swSignalfd_set) {
     ret = sigismember(&curset, SIGUSR1);
     ASSERT_EQ(ret, 0);
 
-    swSignalfd_init();
-    swSignal_set(SIGUSR1, sig_usr1);
-    swSignalfd_setup(SwooleTG.reactor);
+    swoole_signalfd_init();
+    swoole_signal_set(SIGUSR1, sig_usr1);
 
     sigemptyset(&curset);
     sigprocmask(SIG_BLOCK, NULL, &curset);
     ret = sigismember(&curset, SIGUSR1);
     ASSERT_EQ(ret, 1);
 
-    swSignal_set(SIGUSR1, NULL);
-    swSignal_clear();
+    swoole_signal_set(SIGUSR1, NULL);
+    swoole_signal_clear();
 
     sigemptyset(&curset);
     sigprocmask(SIG_BLOCK, NULL, &curset);
