@@ -50,4 +50,4 @@ go(function () {
 	$server->start();
 });
 
-swoole_event_wait();
+Swoole\Event::wait();
