@@ -1,5 +1,5 @@
 <?php
-$serv = new swoole_server("0.0.0.0", 9501);
+$serv = new Swoole\Server("0.0.0.0", 9501);
 $serv->fdlist = [];
 $serv->workerid = 0;
 $serv->set(array(
@@ -18,8 +18,8 @@ $serv->on('workerStart', function($serv, $worker_id) {
 	$serv->workerid = $worker_id;
 });
 
-$serv->on('connect', function ($serv, $fd, $from_id){
-	//echo "[#".posix_getpid()."]\tClient@[$fd:$from_id]: Connect.\n";
+$serv->on('connect', function ($serv, $fd, $reactor_id){
+	//echo "[#".posix_getpid()."]\tClient@[$fd:$reactor_id]: Connect.\n";
 	echo "{$fd} connect, worker:".$serv->workerid.PHP_EOL;
 	$conn = print_r($serv->connection_info($fd));
 	$serv->fdlist[$fd] = 1;
@@ -27,25 +27,25 @@ $serv->on('connect', function ($serv, $fd, $from_id){
 
 });
 
-$serv->on('task', function ($serv, $task_id, $from_id, $data){
-	//var_dump($task_id, $from_id, $data);
+$serv->on('task', function ($serv, $task_id, $reactor_id, $data){
+	//var_dump($task_id, $reactor_id, $data);
 	$fd = $data;
 	$serv->send($fd, str_repeat('B', 1024*rand(40, 60)).rand(10000, 99999)."\n");
 });
 
-$serv->on('finish', function ($serv, $fd, $from_id){
+$serv->on('finish', function ($serv, $fd, $reactor_id){
 
 });
 
-$serv->on('receive', function (swoole_server $serv, $fd, $from_id, $data) {
+$serv->on('receive', function (Swoole\Server $serv, $fd, $reactor_id, $data) {
 
     foreach($serv->fdlist as $_fd=>$val) {
         $serv->send($_fd, "{$fd} say:".$data.PHP_EOL);
     }
 });
 
-$serv->on('close', function ($serv, $fd, $from_id) {
-	//echo "[#".posix_getpid()."]\tClient@[$fd:$from_id]: Close.\n";
+$serv->on('close', function ($serv, $fd, $reactor_id) {
+	//echo "[#".posix_getpid()."]\tClient@[$fd:$reactor_id]: Close.\n";
 	unset($serv->fdlist[$fd]);
 });
 
