@@ -1,12 +1,12 @@
 <?php
-$serv = new swoole_server("127.0.0.1", 9501);
+$serv = new Swoole\Server("127.0.0.1", 9501);
 $serv->set(array(
     'worker_num' => 1,
     'task_worker_num' => 2,
     //'task_tmpdir' => '/data/task/',
 ));
 
-$serv->on('Receive', function(swoole_server $serv, $fd, $reactor_id, $data) {
+$serv->on('Receive', function(Swoole\Server $serv, $fd, $reactor_id, $data) {
 	//AsyncTask
     $data = intval($data);
     for($i=0;$i<$data;$i++) {
@@ -16,7 +16,7 @@ $serv->on('Receive', function(swoole_server $serv, $fd, $reactor_id, $data) {
     }
 
 });
-$serv->on('Task', function (swoole_server $serv, $task_id, $reactor_id, $data) {
+$serv->on('Task', function (Swoole\Server $serv, $task_id, $reactor_id, $data) {
     echo "onTask: [PID=".posix_getpid()."]: task_id=$task_id, data_len=".strlen($data).".".PHP_EOL;
     sleep(10);
     //$serv->finish($data);
@@ -24,11 +24,11 @@ $serv->on('Task', function (swoole_server $serv, $task_id, $reactor_id, $data) {
     return;
 });
 
-$serv->on('Finish', function (swoole_server $serv, $task_id, $data) {
+$serv->on('Finish', function (Swoole\Server $serv, $task_id, $data) {
     echo "Task#$task_id finished, data_len=".strlen($data).PHP_EOL;
 });
 
-$serv->on('Timer', function(swoole_server $serv, $time) {
+$serv->on('Timer', function(Swoole\Server $serv, $time) {
     echo "{$time} call".PHP_EOL;
     print_r($serv->stats());
 });
