@@ -137,14 +137,14 @@ bool HttpContext::parse_multipart_data(const char *at, size_t length) {
     ssize_t n = multipart_parser_execute(mt_parser, at, length);
     if (n < 0) {
         int l_error = multipart_parser_error_msg(mt_parser, sw_tg_buffer()->str, sw_tg_buffer()->size);
-        swoole_error_log(SW_LOG_WARNING,
+        swoole_error_log(SW_LOG_NOTICE,
                          SW_ERROR_SERVER_INVALID_REQUEST,
                          "parse multipart body failed, reason: %.*s",
                          l_error,
                          sw_tg_buffer()->str);
         return false;
     } else if (n != (ssize_t) length) {
-        swoole_error_log(SW_LOG_WARNING,
+        swoole_error_log(SW_LOG_NOTICE,
                          SW_ERROR_SERVER_INVALID_REQUEST,
                          "parse multipart body failed, %lu/%zu bytes processed",
                          n,
@@ -359,15 +359,17 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
     HttpContext *ctx = (HttpContext *) parser->data;
     zval *zheader = ctx->request.zheader;
     size_t header_len = ctx->current_header_name_len;
-    char *header_name = zend_str_tolower_dup(ctx->current_header_name, header_len);
+    zend::CharPtr _header_name;
+    _header_name.assign_tolower(ctx->current_header_name, header_len);
+    char *header_name = _header_name.get();
 
     if (ctx->parse_cookie && SW_STREQ(header_name, header_len, "cookie")) {
         zval *zcookie = swoole_http_init_and_read_property(
             swoole_http_request_ce, ctx->request.zobject, &ctx->request.zcookie, ZEND_STRL("cookie"));
         swoole_http_parse_cookie(zcookie, at, length);
-        efree(header_name);
         return 0;
-    } else if (SW_STREQ(header_name, header_len, "upgrade") && swoole_http_token_list_contains_value(at, length, "websocket")) {
+    } else if (SW_STREQ(header_name, header_len, "upgrade") &&
+               swoole_http_token_list_contains_value(at, length, "websocket")) {
         ctx->websocket = 1;
         if (ctx->co_socket) {
             goto _add_header;
@@ -378,8 +380,7 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
         }
         Connection *conn = serv->get_connection_by_session_id(ctx->fd);
         if (!conn) {
-            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_CLOSED, "session[%ld] is closed", ctx->fd);
-            efree(header_name);
+            swoole_error_log(SW_LOG_TRACE, SW_ERROR_SESSION_CLOSED, "session[%ld] is closed", ctx->fd);
             return -1;
         }
         ListenPort *port = serv->get_port_by_server_fd(conn->server_fd);
@@ -389,9 +390,9 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
     } else if ((parser->method == PHP_HTTP_POST || parser->method == PHP_HTTP_PUT ||
                 parser->method == PHP_HTTP_DELETE || parser->method == PHP_HTTP_PATCH) &&
                SW_STREQ(header_name, header_len, "content-type")) {
-        if (SW_STRCASECT(at, length, "application/x-www-form-urlencoded")) {
+        if (SW_STR_ISTARTS_WITH(at, length, "application/x-www-form-urlencoded")) {
             ctx->request.post_form_urlencoded = 1;
-        } else if (SW_STRCASECT(at, length, "multipart/form-data")) {
+        } else if (SW_STR_ISTARTS_WITH(at, length, "multipart/form-data")) {
             size_t offset = sizeof("multipart/form-data") - 1;
             char *boundary_str;
             int boundary_len;
@@ -407,14 +408,12 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
         ctx->set_compression_method(at, length);
     }
 #endif
-    else if (SW_STREQ(header_name, header_len, "transfer-encoding") && SW_STRCASECT(at, length, "chunked")) {
+    else if (SW_STREQ(header_name, header_len, "transfer-encoding") && SW_STR_ISTARTS_WITH(at, length, "chunked")) {
         ctx->recv_chunked = 1;
     }
 
 _add_header:
     add_assoc_stringl_ex(zheader, header_name, header_len, (char *) at, length);
-    efree(header_name);
-
     return 0;
 }
 
@@ -479,7 +478,9 @@ static int multipart_body_on_header_value(multipart_parser *p, const char *at, s
     }
 
     size_t header_len = ctx->current_header_name_len;
-    char *header_name = zend_str_tolower_dup(ctx->current_header_name, header_len);
+    zend::CharPtr _header_name;
+    _header_name.assign_tolower(ctx->current_header_name, header_len);
+    char *header_name = _header_name.get();
 
     if (SW_STRCASEEQ(header_name, header_len, "content-disposition")) {
         size_t offset = 0;
@@ -489,7 +490,7 @@ static int multipart_body_on_header_value(multipart_parser *p, const char *at, s
             offset += sizeof("attachment;") - 1;
         } else {
             swoole_warning("Unsupported Content-Disposition [%.*s]", (int) length, at);
-            goto _end;
+            return ret;
         }
 
         zval tmp_array;
@@ -498,13 +499,13 @@ static int multipart_body_on_header_value(multipart_parser *p, const char *at, s
 
         zval *zform_name;
         if (!(zform_name = zend_hash_str_find(Z_ARRVAL(tmp_array), ZEND_STRL("name")))) {
-            goto _end;
+            return ret;
         }
 
         if (Z_STRLEN_P(zform_name) >= SW_HTTP_FORM_KEYLEN) {
             swoole_warning("form_name[%s] is too large", Z_STRVAL_P(zform_name));
             ret = -1;
-            goto _end;
+            return ret;
         }
 
         swoole_strlcpy(value_buf, Z_STRVAL_P(zform_name), sizeof(value_buf));
@@ -522,7 +523,7 @@ static int multipart_body_on_header_value(multipart_parser *p, const char *at, s
             if (Z_STRLEN_P(zfilename) >= SW_HTTP_FORM_KEYLEN) {
                 swoole_warning("filename[%s] is too large", Z_STRVAL_P(zfilename));
                 ret = -1;
-                goto _end;
+                return ret;
             }
             ctx->current_input_name = estrndup(tmp, value_len);
             ctx->current_input_name_len = value_len;
@@ -561,10 +562,6 @@ static int multipart_body_on_header_value(multipart_parser *p, const char *at, s
         add_assoc_long(z_multipart_header, "size", swoole::file_get_size(tmp_file.c_str()));
         http_request_add_upload_file(ctx, tmp_file.c_str(), tmp_file.length());
     }
-
-_end:
-    efree(header_name);
-
     return ret;
 }
 
@@ -755,7 +752,9 @@ static int http_request_on_body(swoole_http_parser *parser, const char *at, size
                 length--;
             } while (length != 0);
         }
-        ctx->parse_multipart_data(at, length);
+        if (!ctx->parse_multipart_data(at, length)) {
+            return -1;
+        }
     }
 
     return 0;
