@@ -97,6 +97,9 @@ static void php_swoole_http_response_free_object(zend_object *object) {
     zval ztmp; /* bool, not required to release it */
 
     if (ctx) {
+        if (ctx->onAfterResponse) {
+            ctx->onAfterResponse(ctx);
+        }
         if (!ctx->end_ && (ctx->send_chunked || !ctx->send_header_) && !ctx->detached && sw_reactor()) {
             if (ctx->response.status == 0) {
                 ctx->response.status = SW_HTTP_INTERNAL_SERVER_ERROR;
@@ -281,6 +284,8 @@ static int parse_header_name(const char *key, size_t keylen) {
         return HTTP_HEADER_CONTENT_TYPE;
     } else if (SW_STRCASEEQ(key, keylen, "Transfer-Encoding")) {
         return HTTP_HEADER_TRANSFER_ENCODING;
+    } else if (SW_STRCASEEQ(key, keylen, "Content-Encoding")) {
+        return HTTP_HEADER_CONTENT_ENCODING;
     }
     return 0;
 }
@@ -327,10 +332,9 @@ void HttpContext::build_header(String *http_buffer, const char *body, size_t len
     zval *zheader =
         sw_zend_read_property_ex(swoole_http_response_ce, response.zobject, SW_ZSTR_KNOWN(SW_ZEND_STR_HEADER), 0);
     if (ZVAL_IS_ARRAY(zheader)) {
-        const char *key;
-        uint32_t keylen;
-        int type;
         zval *zvalue;
+        zend_string *string_key;
+        zend_ulong num_key;
 
         auto add_header = [](String *response, const char *key, size_t l_key, zval *value) {
             if (ZVAL_IS_NULL(value)) {
@@ -350,45 +354,55 @@ void HttpContext::build_header(String *http_buffer, const char *body, size_t len
 #ifdef SW_HAVE_COMPRESSION
         zend_string *content_type = nullptr;
 #endif
-        SW_HASHTABLE_FOREACH_START2(Z_ARRVAL_P(zheader), key, keylen, type, zvalue) {
-            // TODO: numeric key name neccessary?
-            if (UNEXPECTED(!key || ZVAL_IS_NULL(zvalue))) {
-                continue;
+        ZEND_HASH_FOREACH_KEY_VAL(Z_ARRVAL_P(zheader), num_key, string_key, zvalue) {
+            if (!string_key) {
+                string_key = zend_long_to_str(num_key);
+            } else {
+                zend_string_addref(string_key);
             }
-
-            int key_header = parse_header_name(key, keylen);
+            zend::String key(string_key, false);
+            int key_header = parse_header_name(ZSTR_VAL(string_key), ZSTR_LEN(string_key));
             if (key_header > 0) {
 #ifdef SW_HAVE_COMPRESSION
                 if (key_header == HTTP_HEADER_CONTENT_TYPE && accept_compression && compression_types) {
                     content_type = zval_get_string(zvalue);
                 }
-
+                if (key_header == HTTP_HEADER_CONTENT_ENCODING && ZVAL_IS_STRING(zvalue) && Z_STRLEN_P(zvalue) == 0) {
+                    accept_compression = 0;
+                }
                 // https://github.com/swoole/swoole-src/issues/4857
+                // https://github.com/swoole/swoole-src/issues/5026
                 if (key_header == HTTP_HEADER_CONTENT_LENGTH && accept_compression) {
-                    php_swoole_error(E_WARNING, "The client has set 'Accept-Encoding', 'Content-Length' is ignored");
+                    swoole_error_log(SW_LOG_WARNING,
+                                     SW_ERROR_HTTP_CONFLICT_HEADER,
+                                     "The client has set 'Accept-Encoding', 'Content-Length' will be ignored");
                     continue;
                 }
 #endif
                 // https://github.com/swoole/swoole-src/issues/4857
+                // https://github.com/swoole/swoole-src/issues/5026
                 if (key_header == HTTP_HEADER_CONTENT_LENGTH && send_chunked) {
-                    php_swoole_error(E_WARNING, "You have set 'Transfer-Encoding', 'Content-Length' is ignored");
+                    swoole_error_log(SW_LOG_WARNING,
+                                     SW_ERROR_HTTP_CONFLICT_HEADER,
+                                     "You have set 'Transfer-Encoding', 'Content-Length' will be ignored");
                     continue;
                 }
-
                 header_flags |= key_header;
+                if (ZVAL_IS_STRING(zvalue) && Z_STRLEN_P(zvalue) == 0) {
+                    continue;
+                }
             }
             if (ZVAL_IS_ARRAY(zvalue)) {
                 zval *zvalue_2;
                 SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(zvalue), zvalue_2) {
-                    add_header(http_buffer, key, keylen, zvalue_2);
+                    add_header(http_buffer, ZSTR_VAL(string_key), ZSTR_LEN(string_key), zvalue_2);
                 }
                 SW_HASHTABLE_FOREACH_END();
             } else {
-                add_header(http_buffer, key, keylen, zvalue);
+                add_header(http_buffer, ZSTR_VAL(string_key), ZSTR_LEN(string_key), zvalue);
             }
         }
-        SW_HASHTABLE_FOREACH_END();
-        (void) type;
+        ZEND_HASH_FOREACH_END();
 
 #ifdef SW_HAVE_COMPRESSION
         if (accept_compression && compression_types) {
@@ -593,7 +607,7 @@ bool HttpContext::compress(const char *data, size_t length) {
         compression_level = Z_BEST_COMPRESSION;
     }
 
-    size_t memory_size = ((size_t)((double) length * (double) 1.015)) + 10 + 8 + 4 + 1;
+    size_t memory_size = ((size_t) ((double) length * (double) 1.015)) + 10 + 8 + 4 + 1;
     zlib_buffer = std::make_shared<String>(memory_size);
 
     z_stream zstream = {};
@@ -850,21 +864,17 @@ _skip_copy:
 }
 
 bool HttpContext::set_header(const char *k, size_t klen, const char *v, size_t vlen, bool format) {
-    zval ztmp;
-    ZVAL_STRINGL(&ztmp, v, vlen);
-    Z_ADDREF(ztmp);
-    return set_header(k, klen, &ztmp, format);
+    zend::Variable ztmp(v, vlen);
+    return set_header(k, klen, ztmp.ptr(), format);
 }
 
 bool HttpContext::set_header(const char *k, size_t klen, zval *zvalue, bool format) {
     if (UNEXPECTED(klen > SW_HTTP_HEADER_KEY_SIZE - 1)) {
         php_swoole_error(E_WARNING, "header key is too long");
-        Z_TRY_DELREF_P(zvalue);
         return false;
     }
 
     if (swoole_http_has_crlf(k, klen)) {
-        Z_TRY_DELREF_P(zvalue);
         return false;
     }
 
@@ -879,7 +889,7 @@ bool HttpContext::set_header(const char *k, size_t klen, zval *zvalue, bool form
         }
         k = sw_tg_buffer()->str;
     }
-    add_assoc_zval_ex(zheader, k, klen, zvalue);
+    zend::array_set(zheader, k, klen, zvalue);
     return true;
 }
 
@@ -985,6 +995,7 @@ static void php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const
         date = php_swoole_format_date((char *) ZEND_STRL("D, d-M-Y H:i:s T"), 1, 0);
         snprintf(cookie, cookie_size, "%s=deleted; expires=%s", name, date);
         efree(date);
+        strlcat(cookie, "; Max-Age=0", cookie_size);
     } else {
         if (url_encode) {
             char *encoded_value;
@@ -1011,6 +1022,18 @@ static void php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const
             }
             strlcat(cookie, date, cookie_size);
             efree(date);
+
+            strlcat(cookie, "; Max-Age=", cookie_size);
+
+            double diff = difftime(expires, php_time());
+            if (diff < 0) {
+                diff = 0;
+            }
+
+            zval max_age;
+            ZVAL_DOUBLE(&max_age, diff);
+            convert_to_string(&max_age);
+            strlcat(cookie, Z_STRVAL_P(&max_age), cookie_size);
         }
     }
     if (path_len > 0) {
@@ -1090,7 +1113,6 @@ static PHP_METHOD(swoole_http_response, header) {
     if (UNEXPECTED(!ctx)) {
         RETURN_FALSE;
     }
-    Z_TRY_ADDREF_P(zvalue);
     RETURN_BOOL(ctx->set_header(k, klen, zvalue, format));
 }
 
