@@ -23,6 +23,7 @@ BEGIN_EXTERN_C()
 #include "stubs/php_swoole_http_server_coro_arginfo.h"
 END_EXTERN_C()
 
+using swoole::Coroutine;
 using swoole::microtime;
 using swoole::PHPCoroutine;
 using swoole::Server;
@@ -54,7 +55,7 @@ class HttpServer {
   public:
     Socket *socket;
     zend_fcall_info_cache *default_handler;
-    std::map<std::string, zend_fcall_info_cache> handlers;
+    std::unordered_map<std::string, zend_fcall_info_cache> handlers;
     zval zcallbacks;
     bool running;
     std::list<Socket *> clients;
@@ -105,8 +106,7 @@ class HttpServer {
         if (pattern == "/") {
             default_handler = &handlers[pattern];
         }
-        Z_ADDREF_P(zcallback);
-        add_assoc_zval_ex(&zcallbacks, pattern.c_str(), pattern.length(), zcallback);
+        zend::array_set(&zcallbacks, pattern.c_str(), pattern.length(), zcallback);
     }
 
     zend_fcall_info_cache *get_handler(HttpContext *ctx) {
@@ -114,7 +114,8 @@ class HttpServer {
             if (&i->second == default_handler) {
                 continue;
             }
-            if (swoole_strcasect(ctx->request.path, ctx->request.path_len, i->first.c_str(), i->first.length())) {
+            if (swoole_str_istarts_with(
+                    ctx->request.path, ctx->request.path_len, i->first.c_str(), i->first.length())) {
                 return &i->second;
             }
         }
@@ -504,6 +505,8 @@ static PHP_METHOD(swoole_http_server_coro, start) {
         hs->upload_tmp_dir = str_v.dup();
     }
 
+    hs->running = true;
+
     while (hs->running) {
         auto conn = sock->accept();
         if (conn) {
@@ -562,12 +565,13 @@ static PHP_METHOD(swoole_http_server_coro, onAccept) {
 
 #ifdef SW_USE_OPENSSL
     if (sock->ssl_is_enable() && !sock->ssl_handshake()) {
-        goto _handshake_failed;
+        RETURN_FALSE;
     }
 #endif
 
     while (true) {
     _recv_request : {
+        sock->get_socket()->recv_wait = 1;
         ssize_t retval = sock->recv(buffer->str + buffer->length, buffer->size - buffer->length);
         if (sw_unlikely(retval <= 0)) {
             break;
@@ -661,6 +665,7 @@ static PHP_METHOD(swoole_http_server_coro, onAccept) {
         zend_fcall_info_cache *fci_cache = hs->get_handler(ctx);
         zval args[2] = {*ctx->request.zobject, *ctx->response.zobject};
         bool keep_alive = swoole_http_should_keep_alive(&ctx->parser) && !ctx->websocket;
+        sock->get_socket()->recv_wait = 0;
 
         if (fci_cache) {
             if (UNEXPECTED(!zend::function::call(fci_cache, 2, args, nullptr, 0))) {
@@ -690,10 +695,6 @@ static PHP_METHOD(swoole_http_server_coro, onAccept) {
         zval_dtor(ctx->request.zobject);
         zval_dtor(ctx->response.zobject);
     }
-
-#ifdef SW_USE_OPENSSL
-_handshake_failed:
-#endif
     /* notice: do not erase the element when server is shutting down */
     if (hs->running) {
         hs->clients.erase(client_iterator);
@@ -706,7 +707,9 @@ static PHP_METHOD(swoole_http_server_coro, shutdown) {
     hs->socket->cancel(SW_EVENT_READ);
     /* accept has been canceled, we only need to traverse once */
     for (auto client : hs->clients) {
-        client->close();
+        if (client->get_socket()->recv_wait) {
+            client->cancel(SW_EVENT_READ);
+        }
     }
     hs->clients.clear();
 }
