@@ -18,6 +18,7 @@
 
 SW_EXTERN_C_BEGIN
 #include "ext/standard/sha1.h"
+#include "stubs/php_swoole_websocket_arginfo.h"
 SW_EXTERN_C_END
 
 #include "swoole_base64.h"
@@ -43,6 +44,8 @@ static zend_object_handlers swoole_websocket_frame_handlers;
 static zend_class_entry *swoole_websocket_closeframe_ce;
 static zend_object_handlers swoole_websocket_closeframe_handlers;
 
+static String *swoole_websocket_buffer = nullptr;
+
 SW_EXTERN_C_BEGIN
 static PHP_METHOD(swoole_websocket_server, push);
 static PHP_METHOD(swoole_websocket_server, isEstablished);
@@ -54,55 +57,21 @@ static PHP_METHOD(swoole_websocket_frame, __toString);
 SW_EXTERN_C_END
 
 // clang-format off
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_push, 0, 0, 2)
-    ZEND_ARG_INFO(0, fd)
-    ZEND_ARG_INFO(0, data)
-    ZEND_ARG_INFO(0, opcode)
-    ZEND_ARG_INFO(0, flags)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_disconnect, 0, 0, 1)
-    ZEND_ARG_INFO(0, fd)
-    ZEND_ARG_INFO(0, code)
-    ZEND_ARG_INFO(0, reason)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_pack, 0, 0, 1)
-    ZEND_ARG_INFO(0, data)
-    ZEND_ARG_INFO(0, opcode)
-    ZEND_ARG_INFO(0, flags)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_unpack, 0, 0, 1)
-    ZEND_ARG_INFO(0, data)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_isEstablished, 0, 0, 1)
-    ZEND_ARG_INFO(0, fd)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_frame_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_swoole_websocket_frame___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
 const zend_function_entry swoole_websocket_server_methods[] =
 {
-    PHP_ME(swoole_websocket_server, push,              arginfo_swoole_websocket_server_push,          ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, disconnect,        arginfo_swoole_websocket_server_disconnect,    ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, isEstablished,     arginfo_swoole_websocket_server_isEstablished, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, pack,              arginfo_swoole_websocket_server_pack,          ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_ME(swoole_websocket_server, unpack,            arginfo_swoole_websocket_server_unpack,        ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_server, push,          arginfo_class_Swoole_WebSocket_Server_push,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, disconnect,    arginfo_class_Swoole_WebSocket_Server_disconnect,    ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, isEstablished, arginfo_class_Swoole_WebSocket_Server_isEstablished, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, pack,          arginfo_class_Swoole_WebSocket_Server_pack,          ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_server, unpack,        arginfo_class_Swoole_WebSocket_Server_unpack,        ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 
 const zend_function_entry swoole_websocket_frame_methods[] =
 {
-    PHP_ME(swoole_websocket_frame, __toString,      arginfo_swoole_websocket_frame___toString,    ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, pack,           arginfo_swoole_websocket_server_pack,   ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_ME(swoole_websocket_server, unpack,         arginfo_swoole_websocket_server_unpack, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_frame, __toString, arginfo_class_Swoole_WebSocket_Frame___toString, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, pack,      arginfo_class_Swoole_WebSocket_Frame_pack,       ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_server, unpack,    arginfo_class_Swoole_WebSocket_Frame_unpack,     ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 // clang-format on
@@ -163,14 +132,15 @@ void php_swoole_websocket_frame_unpack_ex(String *data, zval *zframe, uchar unco
     flags = WebSocket::get_flags(&frame);
 #ifdef SW_HAVE_ZLIB
     if (uncompress && frame.header.RSV1) {
-        swoole_zlib_buffer->clear();
-        if (!websocket_message_uncompress(swoole_zlib_buffer, frame.payload, frame.payload_length)) {
+        String *zlib_buffer = sw_tg_buffer();
+        zlib_buffer->clear();
+        if (!websocket_message_uncompress(zlib_buffer, frame.payload, frame.payload_length)) {
             swoole_set_last_error(SW_ERROR_PROTOCOL_ERROR);
             ZVAL_FALSE(zframe);
             return;
         }
-        frame.payload = swoole_zlib_buffer->str;
-        frame.payload_length = swoole_zlib_buffer->length;
+        frame.payload = zlib_buffer->str;
+        frame.payload_length = zlib_buffer->length;
         flags ^= (WebSocket::FLAG_RSV1 | WebSocket::FLAG_COMPRESS);
     }
 #endif
@@ -215,16 +185,18 @@ static sw_inline int php_swoole_websocket_frame_pack_ex(String *buffer,
         if (!allow_compress) {
             flags ^= WebSocket::FLAG_COMPRESS;
         } else if (length > 0) {
-            swoole_zlib_buffer->clear();
-            if (websocket_message_compress(swoole_zlib_buffer, data, length, Z_DEFAULT_COMPRESSION)) {
-                data = swoole_zlib_buffer->str;
-                length = swoole_zlib_buffer->length;
+            String *zlib_buffer = sw_tg_buffer();
+            zlib_buffer->clear();
+            if (websocket_message_compress(zlib_buffer, data, length, Z_DEFAULT_COMPRESSION)) {
+                data = zlib_buffer->str;
+                length = zlib_buffer->length;
                 flags |= WebSocket::FLAG_RSV1;
             }
         }
     }
 #endif
 
+    // TODO: send data in zlib_buffer directly, do not copy to http_buffer
     switch (opcode) {
     case WebSocket::OPCODE_CLOSE:
         return WebSocket::pack_close_frame(buffer, code, data, length, flags);
@@ -280,10 +252,26 @@ int php_swoole_websocket_frame_object_pack_ex(String *buffer, zval *zdata, zend_
         buffer, zdata, opcode, code, flags & WebSocket::FLAGS_ALL, mask, allow_compress);
 }
 
+void swoole_websocket_onBeforeHandshakeResponse(Server *serv, int server_fd, HttpContext *ctx) {
+    zend_fcall_info_cache *fci_cache =
+        php_swoole_server_get_fci_cache(serv, server_fd, SW_SERVER_CB_onBeforeHandshakeResponse);
+    if (fci_cache) {
+        zval args[3];
+        args[0] = *((zval *) serv->private_data_2);
+        args[1] = *ctx->request.zobject;
+        args[2] = *ctx->response.zobject;
+        if (UNEXPECTED(!zend::function::call(fci_cache, 3, args, nullptr, serv->is_enable_coroutine()))) {
+            php_swoole_error(
+                E_WARNING, "%s->onBeforeHandshakeResponse handler error", ZSTR_VAL(swoole_websocket_server_ce->name));
+            serv->close(ctx->fd, false);
+        }
+    }
+}
+
 void swoole_websocket_onOpen(Server *serv, HttpContext *ctx) {
     Connection *conn = serv->get_connection_by_session_id(ctx->fd);
     if (!conn) {
-        swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_CLOSED, "session[%ld] is closed", ctx->fd);
+        swoole_error_log(SW_LOG_TRACE, SW_ERROR_SESSION_NOT_EXIST, "session[%ld] is closed", ctx->fd);
         return;
     }
     zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, conn->server_fd, SW_SERVER_CB_onOpen);
@@ -356,10 +344,6 @@ bool swoole_websocket_handshake(HttpContext *ctx) {
     ctx->set_header(ZEND_STRL("Sec-WebSocket-Accept"), sec_buf, sec_len, false);
     ctx->set_header(ZEND_STRL("Sec-WebSocket-Version"), ZEND_STRL(SW_WEBSOCKET_VERSION), false);
 
-#ifdef SW_HAVE_ZLIB
-    bool enable_websocket_compression = true;
-    bool websocket_compression = false;
-#endif
     Server *serv = nullptr;
     Connection *conn = nullptr;
 
@@ -367,29 +351,10 @@ bool swoole_websocket_handshake(HttpContext *ctx) {
         serv = (Server *) ctx->private_data;
         conn = serv->get_connection_by_session_id(ctx->fd);
         if (!conn) {
-            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_CLOSED, "session[%ld] is closed", ctx->fd);
+            swoole_error_log(SW_LOG_TRACE, SW_ERROR_SESSION_NOT_EXIST, "session[%ld] is closed", ctx->fd);
             return false;
         }
-#ifdef SW_HAVE_ZLIB
-        enable_websocket_compression = serv->websocket_compression;
-#endif
     }
-#ifdef SW_HAVE_ZLIB
-    else {
-        enable_websocket_compression = ctx->websocket_compression;
-    }
-#endif
-
-#ifdef SW_HAVE_ZLIB
-    if (enable_websocket_compression && (pData = zend_hash_str_find(ht, ZEND_STRL("sec-websocket-extensions"))) &&
-        Z_TYPE_P(pData) == IS_STRING) {
-        std::string value(Z_STRVAL_P(pData), Z_STRLEN_P(pData));
-        if (value.substr(0, value.find_first_of(';')) == "permessage-deflate") {
-            websocket_compression = true;
-            ctx->set_header(ZEND_STRL("Sec-Websocket-Extensions"), ZEND_STRL(SW_WEBSOCKET_EXTENSION_DEFLATE), false);
-        }
-    }
-#endif
 
     if (conn) {
         conn->websocket_status = WebSocket::STATUS_ACTIVE;
@@ -400,9 +365,7 @@ bool swoole_websocket_handshake(HttpContext *ctx) {
                             port->websocket_subprotocol.length(),
                             false);
         }
-#ifdef SW_HAVE_ZLIB
-        ctx->websocket_compression = conn->websocket_compression = websocket_compression;
-#endif
+        swoole_websocket_onBeforeHandshakeResponse(serv, conn->server_fd, ctx);
     } else {
         Socket *sock = (Socket *) ctx->private_data;
         sock->open_length_check = 1;
@@ -410,9 +373,6 @@ bool swoole_websocket_handshake(HttpContext *ctx) {
         sock->protocol.package_length_offset = 0;
         sock->protocol.package_body_offset = 0;
         sock->protocol.get_package_length = WebSocket::get_package_length;
-#ifdef SW_HAVE_ZLIB
-        ctx->websocket_compression = websocket_compression;
-#endif
     }
 
     ctx->response.status = SW_HTTP_SWITCHING_PROTOCOLS;
@@ -585,13 +545,14 @@ int swoole_websocket_onMessage(Server *serv, RecvData *req) {
      * RFC 7692
      */
     if (serv->websocket_compression && (flags & WebSocket::FLAG_RSV1)) {
-        swoole_zlib_buffer->clear();
-        if (!websocket_message_uncompress(swoole_zlib_buffer, Z_STRVAL(zdata), Z_STRLEN(zdata))) {
+        String *zlib_buffer = sw_tg_buffer();
+        zlib_buffer->clear();
+        if (!websocket_message_uncompress(zlib_buffer, Z_STRVAL(zdata), Z_STRLEN(zdata))) {
             zval_ptr_dtor(&zdata);
             return SW_OK;
         }
         zval_ptr_dtor(&zdata);
-        ZVAL_STRINGL(&zdata, swoole_zlib_buffer->str, swoole_zlib_buffer->length);
+        ZVAL_STRINGL(&zdata, zlib_buffer->str, zlib_buffer->length);
         flags ^= (WebSocket::FLAG_RSV1 | WebSocket::FLAG_COMPRESS);
     }
 #endif
@@ -629,7 +590,6 @@ int swoole_websocket_onHandshake(Server *serv, ListenPort *port, HttpContext *ct
 void php_swoole_websocket_server_minit(int module_number) {
     SW_INIT_CLASS_ENTRY_EX(swoole_websocket_server,
                            "Swoole\\WebSocket\\Server",
-                           "swoole_websocket_server",
                            nullptr,
                            swoole_websocket_server_methods,
                            swoole_http_server);
@@ -637,26 +597,16 @@ void php_swoole_websocket_server_minit(int module_number) {
     SW_SET_CLASS_CLONEABLE(swoole_websocket_server, sw_zend_class_clone_deny);
     SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_websocket_server, sw_zend_class_unset_property_deny);
 
-    SW_INIT_CLASS_ENTRY(swoole_websocket_frame,
-                        "Swoole\\WebSocket\\Frame",
-                        "swoole_websocket_frame",
-                        nullptr,
-                        swoole_websocket_frame_methods);
-    #if PHP_VERSION_ID >= 80000
-        zend_class_implements(swoole_websocket_frame_ce, 1, zend_ce_stringable);
-    #endif
+    SW_INIT_CLASS_ENTRY(swoole_websocket_frame, "Swoole\\WebSocket\\Frame", nullptr, swoole_websocket_frame_methods);
+    zend_class_implements(swoole_websocket_frame_ce, 1, zend_ce_stringable);
     zend_declare_property_long(swoole_websocket_frame_ce, ZEND_STRL("fd"), 0, ZEND_ACC_PUBLIC);
     zend_declare_property_string(swoole_websocket_frame_ce, ZEND_STRL("data"), "", ZEND_ACC_PUBLIC);
     zend_declare_property_long(swoole_websocket_frame_ce, ZEND_STRL("opcode"), WebSocket::OPCODE_TEXT, ZEND_ACC_PUBLIC);
     zend_declare_property_long(swoole_websocket_frame_ce, ZEND_STRL("flags"), WebSocket::FLAG_FIN, ZEND_ACC_PUBLIC);
     zend_declare_property_null(swoole_websocket_frame_ce, ZEND_STRL("finish"), ZEND_ACC_PUBLIC);
 
-    SW_INIT_CLASS_ENTRY_EX(swoole_websocket_closeframe,
-                           "Swoole\\WebSocket\\CloseFrame",
-                           "swoole_websocket_closeframe",
-                           nullptr,
-                           nullptr,
-                           swoole_websocket_frame);
+    SW_INIT_CLASS_ENTRY_EX(
+        swoole_websocket_closeframe, "Swoole\\WebSocket\\CloseFrame", nullptr, nullptr, swoole_websocket_frame);
     zend_declare_property_long(
         swoole_websocket_closeframe_ce, ZEND_STRL("opcode"), WebSocket::OPCODE_CLOSE, ZEND_ACC_PUBLIC);
     zend_declare_property_long(
@@ -725,6 +675,17 @@ void php_swoole_websocket_server_minit(int module_number) {
     SW_REGISTER_LONG_CONSTANT("WEBSOCKET_CLOSE_EXTENSION_MISSING", WebSocket::CLOSE_EXTENSION_MISSING);
     SW_REGISTER_LONG_CONSTANT("WEBSOCKET_CLOSE_SERVER_ERROR", WebSocket::CLOSE_SERVER_ERROR);
     SW_REGISTER_LONG_CONSTANT("WEBSOCKET_CLOSE_TLS", WebSocket::CLOSE_TLS);
+
+    if (swoole_websocket_buffer == nullptr) {
+        swoole_websocket_buffer = swoole::make_string(SW_BUFFER_SIZE_BIG);
+    }
+}
+
+void php_swoole_websocket_server_mshutdown() {
+    if (swoole_websocket_buffer) {
+        delete swoole_websocket_buffer;
+        swoole_websocket_buffer = nullptr;
+    }
 }
 
 static sw_inline bool swoole_websocket_server_push(Server *serv, SessionId fd, String *buffer) {
@@ -786,11 +747,11 @@ static PHP_METHOD(swoole_websocket_server, disconnect) {
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|ls", &fd, &code, &data, &length) == FAILURE) {
         RETURN_FALSE;
     }
-    swoole_http_buffer->clear();
-    if (WebSocket::pack_close_frame(swoole_http_buffer, code, data, length, 0) < 0) {
+    swoole_websocket_buffer->clear();
+    if (WebSocket::pack_close_frame(swoole_websocket_buffer, code, data, length, 0) < 0) {
         RETURN_FALSE;
     }
-    RETURN_BOOL(swoole_websocket_server_close(serv, fd, swoole_http_buffer, 1));
+    RETURN_BOOL(swoole_websocket_server_close(serv, fd, swoole_websocket_buffer, 1));
 }
 
 static PHP_METHOD(swoole_websocket_server, push) {
@@ -831,15 +792,15 @@ static PHP_METHOD(swoole_websocket_server, push) {
     allow_compress = conn->websocket_compression;
 #endif
 
-    swoole_http_buffer->clear();
+    swoole_websocket_buffer->clear();
     if (php_swoole_websocket_frame_is_object(zdata)) {
-        if (php_swoole_websocket_frame_object_pack(swoole_http_buffer, zdata, 0, allow_compress) < 0) {
+        if (php_swoole_websocket_frame_object_pack(swoole_websocket_buffer, zdata, 0, allow_compress) < 0) {
             swoole_set_last_error(SW_ERROR_WEBSOCKET_PACK_FAILED);
             RETURN_FALSE;
         }
     } else {
         if (php_swoole_websocket_frame_pack(
-                swoole_http_buffer, zdata, opcode, flags & WebSocket::FLAGS_ALL, 0, allow_compress) < 0) {
+                swoole_websocket_buffer, zdata, opcode, flags & WebSocket::FLAGS_ALL, 0, allow_compress) < 0) {
             swoole_set_last_error(SW_ERROR_WEBSOCKET_PACK_FAILED);
             RETURN_FALSE;
         }
@@ -847,15 +808,33 @@ static PHP_METHOD(swoole_websocket_server, push) {
 
     switch (opcode) {
     case WebSocket::OPCODE_CLOSE:
-        RETURN_BOOL(swoole_websocket_server_close(serv, fd, swoole_http_buffer, flags & WebSocket::FLAG_FIN));
+        RETURN_BOOL(swoole_websocket_server_close(serv, fd, swoole_websocket_buffer, flags & WebSocket::FLAG_FIN));
         break;
     default:
-        RETURN_BOOL(swoole_websocket_server_push(serv, fd, swoole_http_buffer));
+        RETURN_BOOL(swoole_websocket_server_push(serv, fd, swoole_websocket_buffer));
     }
 }
 
+static size_t websocket_get_frame_data_size(zval *zframe) {
+    zend_long opcode = WebSocket::OPCODE_TEXT;
+    zval *ztmp = nullptr;
+    zval *zdata = nullptr;
+
+    if ((ztmp = sw_zend_read_property_ex(swoole_websocket_frame_ce, zframe, SW_ZSTR_KNOWN(SW_ZEND_STR_OPCODE), 1))) {
+        opcode = zval_get_long(ztmp);
+    }
+    if (opcode == WebSocket::OPCODE_CLOSE) {
+        zdata =
+            sw_zend_read_property_not_null_ex(swoole_websocket_frame_ce, zframe, SW_ZSTR_KNOWN(SW_ZEND_STR_REASON), 1);
+    } else {
+        zdata = sw_zend_read_property_ex(swoole_websocket_frame_ce, zframe, SW_ZSTR_KNOWN(SW_ZEND_STR_DATA), 1);
+    }
+    return (zdata && ZVAL_IS_STRING(zdata)) ? Z_STRLEN_P(zdata) : 0;
+}
+
+#define FRAME_HEADER_LEN 16
+
 static PHP_METHOD(swoole_websocket_server, pack) {
-    String *buffer = sw_tg_buffer();
     zval *zdata;
     zend_long opcode = WebSocket::OPCODE_TEXT;
     zval *zflags = nullptr;
@@ -872,17 +851,26 @@ static PHP_METHOD(swoole_websocket_server, pack) {
         flags = zval_get_long(zflags);
     }
 
-    buffer->clear();
+    String *buffer;
+    int result;
     if (php_swoole_websocket_frame_is_object(zdata)) {
-        if (php_swoole_websocket_frame_object_pack(buffer, zdata, 0, 1) < 0) {
-            RETURN_EMPTY_STRING();
-        }
+        buffer =
+            swoole::make_string(websocket_get_frame_data_size(zdata) + FRAME_HEADER_LEN, sw_zend_string_allocator());
+        result = php_swoole_websocket_frame_object_pack(buffer, zdata, 0, 1);
     } else {
-        if (php_swoole_websocket_frame_pack(buffer, zdata, opcode, flags & WebSocket::FLAGS_ALL, 0, 1) < 0) {
-            RETURN_EMPTY_STRING();
-        }
+        buffer = swoole::make_string(Z_STRLEN_P(zdata) + FRAME_HEADER_LEN, sw_zend_string_allocator());
+        result = php_swoole_websocket_frame_pack(buffer, zdata, opcode, flags & WebSocket::FLAGS_ALL, 0, 1);
     }
-    RETURN_STRINGL(buffer->str, buffer->length);
+
+    if (result < 0) {
+        delete buffer;
+        RETURN_EMPTY_STRING();
+    }
+
+    auto packed_str = zend::fetch_zend_string_by_val(buffer->str);
+    ZSTR_VAL(packed_str)[buffer->length] = '\0';
+    ZSTR_LEN(packed_str) = buffer->length;
+    RETURN_STR(packed_str);
 }
 
 static PHP_METHOD(swoole_websocket_frame, __toString) {
