@@ -137,8 +137,8 @@ ssize_t System::write_file(const char *file, char *buf, size_t length, bool lock
 std::string gethostbyname_impl_with_async(const std::string &hostname, int domain, double timeout) {
     AsyncEvent ev{};
 
-    if (hostname.size() < SW_IP_MAX_LENGTH) {
-        ev.nbytes = SW_IP_MAX_LENGTH + 1;
+    if (hostname.size() < INET6_ADDRSTRLEN) {
+        ev.nbytes = INET6_ADDRSTRLEN + 1;
     } else {
         ev.nbytes = hostname.size() + 1;
     }
@@ -391,7 +391,7 @@ static int socket_poll_error_callback(Reactor *reactor, Event *event) {
     return SW_OK;
 }
 
-static int translate_events_to_poll(int events) {
+int translate_events_to_poll(int events) {
     int poll_events = 0;
 
     if (events & SW_EVENT_READ) {
@@ -404,7 +404,7 @@ static int translate_events_to_poll(int events) {
     return poll_events;
 }
 
-static int translate_events_from_poll(int events) {
+int translate_events_from_poll(int events) {
     int sw_events = 0;
 
     if (events & POLLIN) {
