@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   +----------------------------------------------------------------------+
 */
 
@@ -35,7 +35,7 @@ Table *Table::make(uint32_t rows_size, float conflict_proportion) {
         conflict_proportion = SW_TABLE_CONFLICT_PROPORTION;
     }
 
-    Table *table = (Table *) sw_mem_pool()->alloc(sizeof(*table));
+    auto table = (Table *) sw_mem_pool()->alloc(sizeof(Table));
     if (table == nullptr) {
         return nullptr;
     }
@@ -57,9 +57,7 @@ Table *Table::make(uint32_t rows_size, float conflict_proportion) {
 
 void Table::free() {
     delete mutex;
-    if (iterator) {
-        delete iterator;
-    }
+    delete iterator;
     delete column_map;
     delete column_list;
 }
@@ -70,7 +68,7 @@ bool Table::add_column(const std::string &_name, enum TableColumn::Type _type, s
         return false;
     }
 
-    TableColumn *col = new TableColumn(_name, _type, _size);
+    auto col = new TableColumn(_name, _type, _size);
     col->index = item_size;
     item_size += col->size;
     column_map->emplace(_name, col);
@@ -79,10 +77,7 @@ bool Table::add_column(const std::string &_name, enum TableColumn::Type _type, s
     return true;
 }
 
-size_t Table::get_memory_size() {
-    if (memory_size > 0) {
-        return memory_size;
-    }
+size_t Table::calc_memory_size() const {
     /**
      * table size + conflict size
      */
@@ -108,13 +103,15 @@ size_t Table::get_memory_size() {
      */
     _memory_size += size * sizeof(TableRow *);
 
-    memory_size = _memory_size;
-
     swoole_trace("_memory_size=%lu, _row_num=%lu, _row_memory_size=%lu", _memory_size, _row_num, _row_memory_size);
 
     return _memory_size;
 }
 
+size_t Table::get_memory_size() const {
+    return memory_size;
+}
+
 uint32_t Table::get_available_slice_num() {
     lock();
     uint32_t num = pool->get_number_of_spare_slice();
@@ -131,7 +128,7 @@ bool Table::create() {
         return false;
     }
 
-    size_t _memory_size = get_memory_size();
+    size_t _memory_size = calc_memory_size();
     size_t _row_memory_size = sizeof(TableRow) + item_size;
 
     void *_memory = sw_shm_malloc(_memory_size);
@@ -153,6 +150,7 @@ bool Table::create() {
     _memory_size -= _row_memory_size * size;
     pool = new FixedPool(_row_memory_size, _memory, _memory_size, true);
     iterator = new TableIterator(_row_memory_size);
+    memory_size = _memory_size;
     created = true;
 
     return true;
@@ -174,9 +172,7 @@ void Table::destroy() {
     }
     delete column_map;
     delete column_list;
-    if (iterator) {
-        delete iterator;
-    }
+    delete iterator;
     delete pool;
     if (memory) {
         sw_shm_free(memory);
