@@ -173,9 +173,14 @@ int Socket::wait_event(int timeout_ms, int events) {
         if (ret == 0) {
             swoole_set_last_error(SW_ERROR_SOCKET_POLL_TIMEOUT);
             return SW_ERR;
-        } else if (ret < 0 && errno != EINTR) {
-            swoole_sys_warning("poll() failed");
-            return SW_ERR;
+        } else if (ret < 0) {
+            if (errno != EINTR) {
+                swoole_sys_warning("poll() failed");
+            } else if (dont_restart) {
+                return SW_ERR;
+            } else {
+                continue;
+            }
         } else {
             return SW_OK;
         }
@@ -418,7 +423,7 @@ bool Socket::set_buffer_size(uint32_t _buffer_size) {
 
 bool Socket::set_recv_buffer_size(uint32_t _buffer_size) {
     if (set_option(SOL_SOCKET, SO_RCVBUF, _buffer_size) != 0) {
-        swoole_sys_warning("setsockopt(%d, SOL_SOCKET, SO_SNDBUF, %d) failed", fd, _buffer_size);
+        swoole_sys_warning("setsockopt(%d, SOL_SOCKET, SO_RCVBUF, %d) failed", fd, _buffer_size);
         return false;
     }
     return true;
@@ -426,7 +431,7 @@ bool Socket::set_recv_buffer_size(uint32_t _buffer_size) {
 
 bool Socket::set_send_buffer_size(uint32_t _buffer_size) {
     if (set_option(SOL_SOCKET, SO_SNDBUF, _buffer_size) != 0) {
-        swoole_sys_warning("setsockopt(%d, SOL_SOCKET, SO_RCVBUF, %d) failed", fd, _buffer_size);
+        swoole_sys_warning("setsockopt(%d, SOL_SOCKET, SO_SNDBUF, %d) failed", fd, _buffer_size);
         return false;
     }
     return true;
@@ -1101,11 +1106,8 @@ ReturnCode Socket::ssl_accept() {
     } else if (err == SSL_ERROR_SSL) {
         int reason;
         const char *error_string = ssl_get_error_reason(&reason);
-        swoole_warning("bad SSL client[%s:%d], reason=%d, error_string=%s",
-                       info.get_ip(),
-                       info.get_port(),
-                       reason,
-                       error_string ? error_string : "(none)");
+        swoole_warning(
+            "bad SSL client[%s:%d], reason=%d, error_string=%s", info.get_ip(), info.get_port(), reason, error_string);
         return SW_ERROR;
     } else if (err == SSL_ERROR_SYSCALL) {
 #ifdef SW_SUPPORT_DTLS
@@ -1158,11 +1160,12 @@ int Socket::ssl_connect() {
     }
 
     long err_code = ERR_get_error();
-    ERR_error_string_n(err_code, sw_tg_buffer()->str, sw_tg_buffer()->size);
-    swoole_notice("connect to SSL server[%s:%d] failed. Error: %s[%ld|%d]",
+    char *msg = ERR_error_string(err_code, sw_tg_buffer()->str);
+    swoole_notice("Socket::ssl_connect(fd=%d) to server[%s:%d] failed. Error: %s[%ld|%d]",
+                  fd,
                   info.get_ip(),
                   info.get_port(),
-                  sw_tg_buffer()->str,
+                  msg,
                   err,
                   ERR_GET_REASON(err_code));
 
@@ -1455,6 +1458,9 @@ int Socket::ssl_create(SSLContext *ssl_context, int _flags) {
 
 using network::Socket;
 
+/**
+ * return nullptr when fail to create socket, read errno to get failure reason
+ */
 Socket *make_socket(SocketType type, FdType fd_type, int flags) {
     int sock_domain;
     int sock_type;
