@@ -336,19 +336,19 @@ int Server::start_check() {
     if (is_process_mode()) {
         if (!is_support_unsafe_events()) {
             if (onConnect) {
-                swoole_warning("cannot set 'onConnect' event when using dispatch_mode=1/3/7");
+                swoole_warning("cannot set 'onConnect' event when using dispatch_mode=%d", dispatch_mode);
                 onConnect = nullptr;
             }
             if (onClose) {
-                swoole_warning("cannot set 'onClose' event when using dispatch_mode=1/3/7");
+                swoole_warning("cannot set 'onClose' event when using dispatch_mode=%d", dispatch_mode);
                 onClose = nullptr;
             }
             if (onBufferFull) {
-                swoole_warning("cannot set 'onBufferFull' event when using dispatch_mode=1/3/7");
+                swoole_warning("cannot set 'onBufferFull' event when using dispatch_mode=%d", dispatch_mode);
                 onBufferFull = nullptr;
             }
             if (onBufferEmpty) {
-                swoole_warning("cannot set 'onBufferEmpty' event when using dispatch_mode=1/3/7");
+                swoole_warning("cannot set 'onBufferEmpty' event when using dispatch_mode=%d", dispatch_mode);
                 onBufferEmpty = nullptr;
             }
             disable_notify = 1;
@@ -600,7 +600,7 @@ void Server::call_worker_start_callback(Worker *worker) {
         call_hook(Server::HOOK_WORKER_START, hook_args);
     }
     if (onWorkerStart) {
-        onWorkerStart(this, worker->id);
+        onWorkerStart(this, worker);
     }
 }
 
@@ -1530,8 +1530,8 @@ bool Server::sendwait(SessionId session_id, const void *data, uint32_t length) {
     Connection *conn = get_connection_verify(session_id);
     if (!conn) {
         swoole_error_log(SW_LOG_TRACE,
-                         SW_ERROR_SESSION_CLOSED,
-                         "send %d byte failed, because session#%ld is closed",
+                         SW_ERROR_SESSION_NOT_EXIST,
+                         "send %d byte failed, because session#%ld is not exists",
                          length,
                          session_id);
         return false;
@@ -1768,6 +1768,7 @@ ListenPort *Server::add_port(SocketType type, const char *host, int port) {
     ls->socket = make_socket(
         ls->type, ls->is_dgram() ? SW_FD_DGRAM_SERVER : SW_FD_STREAM_SERVER, SW_SOCK_CLOEXEC | SW_SOCK_NONBLOCK);
     if (ls->socket == nullptr) {
+        swoole_set_last_error(errno);
         return nullptr;
     }
 #if defined(SW_SUPPORT_DTLS) && defined(HAVE_KQUEUE)
@@ -1777,6 +1778,7 @@ ListenPort *Server::add_port(SocketType type, const char *host, int port) {
 #endif
 
     if (ls->socket->bind(ls->host, &ls->port) < 0) {
+        swoole_set_last_error(errno);
         ls->socket->free();
         return nullptr;
     }
