@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   +----------------------------------------------------------------------+
 */
 
@@ -29,6 +29,14 @@ bool swoole_timer_is_available() {
     return SwooleTG.timer != nullptr;
 }
 
+TimerNode *swoole_timer_add(double timeout, bool persistent, const TimerCallback &callback, void *private_data) {
+    if (timeout < SW_TIMER_MIN_SEC) {
+        return swoole_timer_add(1L, persistent, callback, private_data);
+    }
+
+    return swoole_timer_add((long) (timeout * 1000), persistent, callback, private_data);
+}
+
 TimerNode *swoole_timer_add(long ms, bool persistent, const TimerCallback &callback, void *private_data) {
     if (sw_unlikely(!swoole_timer_is_available())) {
         SwooleTG.timer = new Timer();
