@@ -4,13 +4,13 @@ require_once dirname(dirname(__DIR__)).'/bootstrap.php';
 class TcpServer
 {
     /**
-     * @var \swoole_server
+     * @var Swoole\Server
      */
     public $swooleServer;
 
     public function __construct()
     {
-        $this->swooleServer = new \swoole_server('127.0.0.1', 9001, SWOOLE_PROCESS, SWOOLE_SOCK_TCP);
+        $this->swooleServer = new Swoole\Server('127.0.0.1', 9001, SWOOLE_PROCESS, SWOOLE_SOCK_TCP);
         $this->swooleServer->set([
             // "output_buffer_size" => 1024 * 1024 * 1024, // 输出限制
 
@@ -55,32 +55,32 @@ class TcpServer
 //        print("closing .....");
     }
 
-    public function onStart(swoole_server $swooleServer)
+    public function onStart(Swoole\Server $swooleServer)
     {
 //        print("swoole_server starting .....");
     }
 
-    public function onShutdown(swoole_server $swooleServer)
+    public function onShutdown(Swoole\Server $swooleServer)
     {
 //        print("swoole_server shutdown .....");
     }
 
-    public function onWorkerStart(swoole_server $swooleServer, $workerId)
+    public function onWorkerStart(Swoole\Server $swooleServer, $workerId)
     {
 //        print("worker #$workerId starting .....");
     }
 
-    public function onWorkerStop(swoole_server $swooleServer, $workerId)
+    public function onWorkerStop(Swoole\Server $swooleServer, $workerId)
     {
 //        print("worker #$workerId stopping ....");
     }
 
-    public function onWorkerError(swoole_server $swooleServer, $workerId, $workerPid, $exitCode, $sigNo)
+    public function onWorkerError(Swoole\Server $swooleServer, $workerId, $workerPid, $exitCode, $sigNo)
     {
 //        print("worker error happening [workerId=$workerId, workerPid=$workerPid, exitCode=$exitCode, signalNo=$sigNo]...");
     }
 
-    public function onReceive(swoole_server $swooleServer, $fd, $fromId, $data)
+    public function onReceive(Swoole\Server $swooleServer, $fd, $fromId, $data)
     {
         static $i;
         if ($i > USE_VALGRIND ? 200 : 20000)
