@@ -75,7 +75,7 @@
 #define CR 13
 
 enum state {
-    s_uninitialized = 1,
+    s_uninitialized = 0,
     s_start,
     s_start_boundary,
     s_header_field_start,
@@ -88,7 +88,6 @@ enum state {
     s_part_data,
     s_part_data_almost_boundary,
     s_part_data_boundary,
-    s_part_data_almost_almost_end,
     s_part_data_almost_end,
     s_part_data_end,
     s_part_data_final_hyphen,
@@ -99,9 +98,9 @@ multipart_parser *multipart_parser_init(const char *boundary,
                                         size_t boundary_length,
                                         const multipart_parser_settings *settings) {
     multipart_parser *p = calloc(sizeof(multipart_parser) + boundary_length + boundary_length + 9 + 4, sizeof(char));
-    memcpy(p->multipart_boundary, "--", 2);
-    memcpy(p->multipart_boundary + 2, boundary, boundary_length);
-    p->multipart_boundary[2 + boundary_length] = 0;
+    memcpy(p->boundary, "--", 2);
+    memcpy(p->boundary + 2, boundary, boundary_length);
+    p->boundary[2 + boundary_length] = 0;
 
     p->boundary_length = boundary_length + 2;
     p->index = 0;
@@ -120,7 +119,70 @@ void multipart_parser_free(multipart_parser *p) {
     free(p);
 }
 
-size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len) {
+int multipart_parser_error_msg(multipart_parser *p, char *buf, size_t len) {
+    int ret;
+    switch (p->error_reason) {
+    case MPPE_OK:
+        return 0;
+    case MPPE_PAUSED:
+        return snprintf(buf, len, "parser paused");
+    case MPPE_UNKNOWN:
+    default:
+        abort();
+        return 0;
+    case MPPE_BOUNDARY_END_NO_CRLF:
+        ret = snprintf(buf, len, "no CRLF at first boundary end: ");
+        break;
+    case MPPE_BAD_START_BOUNDARY:
+        ret = snprintf(buf, len, "first boundary mismatching: ");
+        break;
+    case MPPE_INVALID_HEADER_FIELD_CHAR:
+        ret = snprintf(buf, len, "invalid char in header field: ");
+        break;
+    case MPPE_INVALID_HEADER_VALUE_CHAR:
+        ret = snprintf(buf, len, "invalid char in header value: ");
+        break;
+    case MPPE_BAD_PART_END:
+        ret = snprintf(buf, len, "no next part or final hyphen: expecting CR or '-' ");
+        break;
+    case MPPE_END_BOUNDARY_NO_DASH:
+        ret = snprintf(buf, len, "bad final hyphen: ");
+        break;
+    }
+    if (ret < 0) {
+        return 0;
+    }
+    if ((size_t) ret >= len) {
+        return ret;
+    }
+    switch (p->error_expected) {
+    case '\0':
+        break;
+    case CR:
+        ret += snprintf(buf + ret, len - ret, "expecting CR ");
+        break;
+    case LF:
+        ret += snprintf(buf + ret, len - ret, "expecting LF ");
+        break;
+    default:
+        ret += snprintf(buf + ret, len - ret, "expecting '%c' ", p->error_expected);
+        break;
+    }
+    if (ret < 0) {
+        return 0;
+    }
+    if ((size_t) ret >= len) {
+        return ret;
+    }
+    if (isprint(p->error_unexpected)) {
+        ret += snprintf(buf + ret, len - ret, "at %zu, but it is '%c'", p->error_i, p->error_unexpected);
+    } else {
+        ret += snprintf(buf + ret, len - ret, "at %zu, but it is '\\x%.2x'", p->error_i, p->error_unexpected);
+    }
+    return ret;
+}
+
+ssize_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len) {
     size_t i = 0;
     size_t mark = 0;
     size_t mark_end = 0;
@@ -137,6 +199,7 @@ size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len
             p->index = 0;
             p->state = s_start_boundary;
             /* fallthrough */
+            /* no break */
         case s_start_boundary:
             multipart_log_c("s_start_boundary");
             if (p->index == p->boundary_length) {
@@ -154,8 +217,8 @@ size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len
                 NOTIFY_CB(part_data_begin, i + 1);
                 break;
             }
-            if (c != p->multipart_boundary[p->index]) {
-                ERROR_EXPECT(MPPE_BAD_START_BOUNDARY, p->multipart_boundary[p->index]);
+            if (c != p->boundary[p->index]) {
+                ERROR_EXPECT(MPPE_BAD_START_BOUNDARY, p->boundary[p->index]);
             }
             p->index++;
             break;
@@ -164,6 +227,7 @@ size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len
             mark = i;
             p->state = s_header_field;
             /* fallthrough */
+            /* no break */
         case s_header_field:
             multipart_log_c("s_header_field");
             if (c == CR) {
@@ -206,14 +270,14 @@ size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len
             mark = i;
             p->state = s_header_value;
             /* fallthrough */
+            /* no break */
         case s_header_value:
             multipart_log_c("s_header_value");
             if (c == CR) {
                 p->state = s_header_value_almost_done;
                 EMIT_DATA_CB(header_value, i + 1, buf + mark, i - mark);
-            }
-            if (is_last) {
-                EMIT_DATA_CB(header_value, i + 1, buf + mark, i - mark + 1);
+            } else if (is_last) {
+                ERROR_EXPECT(MPPE_HEADER_VALUE_INCOMPLETE, CR);
             }
             break;
         case s_header_value_almost_done:
@@ -229,20 +293,16 @@ size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len
             p->state = s_part_data;
             NOTIFY_CB(headers_complete, i);
             /* fallthrough */
+            /* no break */
         case s_part_data:
         data_rollback:
             multipart_log_c("s_part_data");
             mark_end = i + 1;
             if (c == CR) {
-                mark_end--;
-                if (is_last) {
-                    if (i > 1) {
-                        EMIT_DATA_CB(part_data, i, buf + mark, mark_end - mark);
-                    } else {
-                        // donot trig callback
-                        return 0;
-                    }
+                if (mark_end - mark - 1 > 0) {
+                    EMIT_DATA_CB(part_data, i + 1, buf + mark, mark_end - mark - 1);
                 }
+                mark = i;
                 p->state = s_part_data_almost_boundary;
                 break;
             }
@@ -252,53 +312,43 @@ size_t multipart_parser_execute(multipart_parser *p, const char *buf, size_t len
             break;
         case s_part_data_almost_boundary:
             multipart_log_c("s_part_data_almost_boundary");
-            if (c == LF) {
-                if (is_last) {
-                    if (i > 2) {
-                        EMIT_DATA_CB(part_data, mark_end, buf + mark, mark_end - mark);
-                    } else {
-                        // donot trig callback
-                        return 0;
-                    }
-                }
+            if (c != LF) {
+                EMIT_DATA_CB(part_data, i + 1, "\r", 1);
+                p->state = s_part_data;
+                mark = i;
+                goto data_rollback;
+            } else {
                 p->state = s_part_data_boundary;
                 p->index = 0;
                 break;
             }
-            p->state = s_part_data;
-            goto data_rollback;
         case s_part_data_boundary:
             multipart_log_c("s_part_data_boundary");
-            if (p->multipart_boundary[p->index] != c) {
+            if (p->boundary[p->index] != c) {
+                EMIT_DATA_CB(part_data, i + 1, "\r\n", 2);
+                if (p->index > 0) {
+                    EMIT_DATA_CB(part_data, i + 1, p->boundary, p->index);
+                }
+                mark = i;
                 p->state = s_part_data;
                 goto data_rollback;
-            }
-            if (is_last) {
-                if (i > p->index + 2) {
-                    EMIT_DATA_CB(part_data, i - p->index - 2, buf + mark, mark_end - mark);
-                } else {
-                    // donot trig callback
-                    return 0;
+            } else {
+                p->index++;
+                if (p->index == p->boundary_length) {
+                    p->state = s_part_data_almost_end;
                 }
+                break;
             }
-            if ((++p->index) == p->boundary_length) {
-                p->state = s_part_data_almost_almost_end;
-                EMIT_DATA_CB(part_data, i + 1, buf + mark, i + 1 - p->boundary_length - 2 - mark);
-            }
-            break;
-        case s_part_data_almost_almost_end:
-            multipart_log_c("s_part_data_almost_almost_end");
-            p->state = s_part_data_almost_end;
-            NOTIFY_CB(part_data_end, i);
-            /* fallthrough */
         case s_part_data_almost_end:
             multipart_log_c("s_part_data_almost_end");
             if (c == '-') {
                 p->state = s_part_data_final_hyphen;
+                NOTIFY_CB(part_data_end, i + 1);
                 break;
             }
             if (c == CR) {
                 p->state = s_part_data_end;
+                NOTIFY_CB(part_data_end, i + 1);
                 break;
             }
             // should be end or another part
